/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.area.field;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPolygon;
import java.beans.IntrospectionException;
import java.util.Collection;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import nl.cloudfarming.client.area.AreaService;
import nl.cloudfarming.client.area.field.explorer.GeometryRootNode;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.model.*;
import nl.cloudfarming.client.util.GeometryUtil;
import org.openide.explorer.ExplorerManager;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Timon Veenstra
 */
@ServiceProvider(service = FieldAreaService.class)
public final class FieldAreaService extends CommonModelService implements AreaService {
    
    private final ConcurrentMap<Long, Field> fieldMap = new ConcurrentHashMap<Long, Field>();
    private final ConcurrentMap<Long, HelpLine> helpLineMap = new ConcurrentHashMap<Long, HelpLine>();
    
    //FIXME
    private static long globalId = 0;

    @Override
    final public Collection<Field> findAllCurrentFields() {
        return fieldMap.values();
        //TODO initial fetch from DataThread Field.Q_FIND_ALL_VALID_FIELDS
    }

    @Override
    final public Field createNewField(String name, MultiPolygon geometry, String code) {
        Field field = new Field();
        field.setName(name);
        field.setGeometry(geometry);
        field.setCode(code);
        field.setAcreage(GeometryUtil.getAcreageInHA(GeometryUtil.getAcreageFromGeometry(geometry)));
        
        //TODO get id from id generator and set it on the field, probably better to return field based on value paramaters
        fieldMap.put(globalId++, field);
        return field;
    }

    /**
     * Sets the endDate of the field to now.
     *
     * @param field the field that should be ended @retrun boolean true if
     * succesfull false if not succesfull
     * @throws IllegalArgumentException if the Provided field is not a persisted
     * entity
     */
    @Override
    public boolean endField(Field field) {
        Date now = new Date();

        if (field.getStartDate() == null || field.getStartDate().before(now)) {
            field.setEndDate(now);
            return true;
        }
        return false;
    }

    @Override
    protected ExplorerManager initExplorer() {
        try {
            ExplorerManager manager = new ExplorerManager();
            //manager.setRootContext(new FieldRootNode(new FieldLayer("service.explorer.layerlist.field_node.name", this)));

            FieldLayer fieldLayer = new FieldLayer("service.explorer.layerlist.field_node.name");
            Layer.Sync.SyncBaseExplorer(fieldLayer, manager);

            HelpLineLayer lineLayer = new HelpLineLayer("service.explorer.layerlist.line_node.name");
            Layer.Sync.SyncBaseExplorer(lineLayer, manager);

            manager.setRootContext(new GeometryRootNode(fieldLayer, lineLayer));


            manager.getRootContext().setDisplayName(NbBundle.getMessage(this.getClass(), "service.explorer.rootnode.displayname"));


            
            Layer.Sync.SyncBaseExplorer(fieldLayer, manager);
            Layer.Sync.SyncBaseExplorer(lineLayer, manager);
            
            return manager;
        } catch (IntrospectionException ex) {
            Exceptions.printStackTrace(ex);
            throw new RuntimeException(ex);
        }
    }

    /**
     * Find all the fields where the endDate is not filled
     *
     * @return all the active Fields
     */
    @Override
    public Collection<HelpLine> findAllHelpLines() {
        return helpLineMap.values();
    }

    /**
     * Removes a field. If a field has partFields the field will not be removed.
     * Instead the endDate of the field and all of its partFields will be filed.
     *
     * @param field which will be removed
     */
    @Override
    public void removeField(Field field) {
        endField(field);
        field.setEndDate(new Date());
        fieldMap.remove(field.getId());
    }

    /**
     * Removes a line. If a field has partFields the field will not be removed.
     * Instead the endDate of the field and all of its partFields will be filed.
     *
     * @param field which will be removed
     */
    @Override
    public void removeHelpLine(HelpLine line) {
        helpLineMap.remove(line.getId());
    }

    @Override
    public void createNewHelpLine(String name, MultiLineString geometry, String NAME) {
        HelpLine helpLine = new HelpLine();
        helpLine.setName(name);
        helpLine.setGeometry(geometry);
        
        //TODO get id from id generator and set it on the field, probably better to return field based on value paramaters
        helpLineMap.put(globalId++, helpLine);
    }
}
